pub struct PuffContext { /* private fields */ }
The central control structure for dispatching tasks onto coroutine workers. All tasks in the same runtime will have access to the same dispatcher. The dispatcher contains a reference to the parent Tokio Runtime as well as references to all coroutine workers.

Implementations

Creates an empty RuntimeDispatcher with no active threads for testing.

Creates a new RuntimeDispatcher using the supplied RuntimeConfig.

Creates a new RuntimeDispatcher using the supplied RuntimeConfig. This function will start the number of coroutine_threads specified in your config. Includes options.

A Handle into the multi-threaded async runtime

The global configured PubSubClient

The global configured reqwest::Client

The global configured PubSubClient

A Handle into the multi-threaded async runtime

The configured redis client. Panics if not enabled.

The configured postgres client. Panics if not enabled.

The configured graphql root node. Panics if not enabled.