Type Definition puff_rs::context::PuffContext

source ·
pub type PuffContext = Arc<RealPuffContext>;
The central control structure for dispatching tasks onto coroutine workers. All tasks in the same runtime will have access to the same dispatcher. The dispatcher contains a reference to the parent Tokio Runtime as well as references to all coroutine workers.