Function puff_rs::errors::log_puff_error
source · pub fn log_puff_error<T>(label: &str, r: PuffResult<T>) -> PuffResult<T>Expand description
Log a puff error if it exists for the result, and return the result back.
pub fn log_puff_error<T>(label: &str, r: PuffResult<T>) -> PuffResult<T>Log a puff error if it exists for the result, and return the result back.