Attribute Macro puff_rs::python::pyclass

#[pyclass]
A proc macro used to expose Rust structs and fieldless enums as Python objects.

For more on creating Python classes, see the class section of the guide.