Struct puff_rs::python::PythonDispatcher
source · [−]pub struct PythonDispatcher { /* private fields */ }Implementations
sourceimpl PythonDispatcher
impl PythonDispatcher
pub fn new(context_waiting: Arc<Mutex<Option<PuffContext>>>) -> PyResult<Self>
pub fn blocking() -> PyResult<Self>
sourcepub fn dispatch_blocking<A: IntoPy<Py<PyTuple>>, K: IntoPy<Py<PyDict>>>(
&self,
py: Python<'_>,
function: PyObject,
args: A,
kwargs: K
) -> PyResult<Receiver<PyResult<PyObject>>>
pub fn dispatch_blocking<A: IntoPy<Py<PyTuple>>, K: IntoPy<Py<PyDict>>>(
&self,
py: Python<'_>,
function: PyObject,
args: A,
kwargs: K
) -> PyResult<Receiver<PyResult<PyObject>>>
Run the python function a new Tokio blocking working thread.
sourcepub fn dispatch1<A: IntoPy<Py<PyTuple>> + Send + 'static>(
&self,
function: PyObject,
args: A
) -> PyResult<Receiver<PyResult<PyObject>>>
pub fn dispatch1<A: IntoPy<Py<PyTuple>> + Send + 'static>(
&self,
function: PyObject,
args: A
) -> PyResult<Receiver<PyResult<PyObject>>>
Acquires the GIL and Executes the python function on the greenlet thread or a new thread depending if greenlets were enabled. Takes no keyword arguments