Module puff_rs::runtime

source ·
Types used to interact with the Puff Runtime

Structs

Controls options for running puff tasks.

Enums

Strategies for distributing tasks onto a coroutine worker thread. In general the strategy shouldn’t matter that much if your tasks are roughly uniform in terms of computational requirements. However, some strategies might be better depending on the context of your application. For example, Strategy::LeastBusy can be useful if you have certain tasks that might be heavier than others. All counting and distributions are done with atomic counters for speed, however there may be some differences in practice from the described as atomic counters can be concurrently read.